/*
 * ISHELL: detection
 *
 * by: Arnaud Jacques (scrap)
 * http://www.securiteinfo.com / webmaster@securiteinfo.com
 *
 * based on the great work of: Peter Kieltyka (elux)
 * http://peter.eluks.com / peter@eluks.com
 *
 */

#include "ishell.h"

/* function prototypes */
void    usage(char *);
void    ish_timeout(int);
int     ish_prompt(int, struct sockaddr *, socklen_t);

int     sockfd;
struct  sockaddr_in sin2;
struct itimerval timer, otimer;

void
usage(char *program) {
    fprintf(stderr,
    "usage: %s -t <timeout> <host>\n\n"
    "options:\n"
    " -t <timeout>    Set reception timeout in microseconds (default: 500000)\n"
    "suggested values:\n"
    " 100 Mb LAN : 10000\n"
    " 10 Mb LAN  : 20000\n"
    " Internet   : 500000\n"
    "\nexample:\n"
    "%s -t 100000 host.com\n"
    "\n", program, program);

    exit(-1);
}

void
ish_timeout(int empty) {
    printf(".");
    setvbuf(stdout, NULL, _IONBF, 0);
    fflush(stdout);

    if(ish_send(sockfd, "id\n", (struct sockaddr *)&sin2, sizeof(sin2)) < 0) {
        printf(" failed.\n\n");
        error_msg();
    }

    ish_info.id++;
    setitimer(ITIMER_REAL, &timer, &otimer);

    return;
}

int
main(int argc, char *argv[]) {
    char    *host, opt;
    struct  hostent *he;

    timer.it_value.tv_usec = 500000L;

    printf("\nICMP Shell Detection v%s  -  by: Arnaud Jacques (webmaster@securiteinfo.com)\n", VERSION);
    printf("-----------------------------------------------------------------------------\n");
    if(argc < 2)
        usage(argv[0]);

    while((opt = getopt(argc, argv, "t:")) != -1) {
        switch(opt) {
            case 't':
                timer.it_value.tv_usec = atoi(optarg);
                break;
            default:
                usage(argv[0]);
                break;
        }
    }

    ish_info.id = 0;

    host = argv[argc-1];

    if((he = gethostbyname(host)) == NULL) {
        fprintf(stderr, "Error: Cannot resolve %s!\n", host);
        exit(-1);
    }

    sin2.sin_family = AF_INET;
    sin2.sin_addr = *((struct in_addr *)he->h_addr);
    memset(&(sin2.sin_zero), 0, 8);

    if((sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) == -1)
        error_msg();

    sendhdr.cntrl = 0;
    sendhdr.cntrl |= CNTRL_CPOUT;

    timer.it_interval.tv_sec = 0;
    timer.it_interval.tv_usec = 0;
    timer.it_value.tv_sec = 0;

    signal(SIGALRM, ish_timeout);
    setitimer(ITIMER_REAL, &timer, &otimer);

    for (;;)
    {
        if(ish_recv(sockfd,NULL, (struct sockaddr *)&sin2, sizeof(sin2)) != 0) {
           printf("\nFound on id %i.\n\n", ish_info.id-1);
           close(sockfd);
           exit(0);
        }
    }
}
